@@ -118,26 +118,34 @@
 #' @rdname microaggregation
 #' @export
 #' @examples
-#' data(Tarragona)
-#' m1 <- microaggregation(Tarragona, method='onedims', aggr=3)
-#' ## summary(m1)
 #' data(testdata)
-#' m2 <- microaggregation(testdata[1:100,c('expend','income','savings')],
-#'   method='mdav', aggr=4)
-#' summary(m2)
+#' m <- microaggregation(
+#'   obj = testdata[1:100, c("expend", "income", "savings")],
+#'   method = "mdav",
+#'   aggr = 4
+#' )
+#' summary(m)
 #'
 #' ## for objects of class sdcMicro:
-#' ## no stratification because @strataVar is NULL
+#' ## no stratification because `@strataVar` is `NULL`
 #' data(testdata2)
-#' sdc <- createSdcObj(testdata2,
-#'   keyVars=c('urbrur','roof','walls','water','electcon','sex'),
-#'   numVars=c('expend','income','savings'), w='sampling_weight')
-#' sdc <- microaggregation(sdc, variables=c("expend","income"))
+#' sdc <- createSdcObj(
+#'   dat = testdata2,
+#'   keyVars = c("urbrur", "roof", "walls", "water", "electcon", "sex"),
+#'   numVars = c("expend", "income", "savings"),
+#'   w = "sampling_weight"
+#' )
+#' sdc <- microaggregation(
+#'   obj = sdc,
+#'   variables = c("expend", "income")
+#' )
 #'
-#' ## with stratification by 'relat'
+#' ## with stratification using variable `"relat"`
 #' strataVar(sdc) <- "relat"
-#' sdc <- microaggregation(sdc, variables=c("savings"))
-
+#' sdc <- microaggregation(
+#'   obj = sdc,
+#'   variables = "savings"
+#' )
 microaggregation <- function(obj, variables=NULL, aggr=3, strata_variables=NULL,
   method="mdav", weights=NULL, nc=8, clustermethod="clara",
   measure="mean", trim=0, varsort=1, transf="log") {
@@ -693,7 +701,6 @@ microaggregationWORK <- function(x, variables=colnames(x), method="mdav", aggr=3
     y <- apply(y, 2, function(x) (x - mean(x, na.rm=TRUE))/sd(x, na.rm=TRUE))
 
     d <- as.matrix(stats::dist(y))
-    set.seed(123)
     rr <- covMcd(y)
     md <- stats::mahalanobis(y, center=rr$center, cov=rr$cov)
     diag(d) <- 0
@@ -897,10 +904,11 @@ print.micro <- function(x, ...) {
 #' @method summary micro
 #' @export
 #' @examples
-#'
 #' data(Tarragona)
-#' m1 <- microaggregation(Tarragona, method='onedims', aggr=3)
-#' ## summary(m1)
+#' m1 <- microaggregation(Tarragona, method = "onedims", aggr = 3)
+#' \donttest{
+#' summary(m1)
+#' }
 summary.micro <- function(object, ...) {
   prcompRob <- function(X, k=0, sca="mad", scores=TRUE) {
     ## Copyright: Croux and Filzmoser
